from urllib.parse import unquote
from http.cookies import SimpleCookie
from email.parser import BytesParser as BP
from orjson import loads


class Request:
	def __init__(self, req):
		self._headers = req
		self.host = req.get("HTTP_HOST", "")
		self.method = req["REQUEST_METHOD"]
		self.path = req["PATH_INFO"]
		self.query_string = Request._parse_qsa(unquote(req["QUERY_STRING"]).split("&"))
		self.data = {}
		self.content_type = req.get("CONTENT_TYPE")
		self.content_length = int(req.get("CONTENT_LENGTH") or 0)
		self._bad_request = False
		self.storage = {}
		self.cookie = SimpleCookie()
		if raw_cookie := req.get("HTTP_COOKIE"):
			self.cookie.load(raw_cookie)
		if req.get("HTTP_X_FORWARDED_FOR"):
			self.ip = req["HTTP_X_FORWARDED_FOR"].split(',')[-1].strip()
		else:
			self.ip = req["REMOTE_ADDR"]

		if self.content_length:
			try:
				if self.content_type[:16] == "application/json":
					self.data = loads(req["wsgi.input"].read(self.content_length))
				elif self.content_type == "application/x-www-form-urlencoded":
					self.data = Request._parse_qsa(unquote(req["wsgi.input"].read(self.content_length).decode()).split("&"))
				elif self.content_type[:20] == "multipart/form-data;":
					m = BP().parsebytes(b"Content-Type: " + self.content_type.encode() + b"\n" + req["wsgi.input"].read(self.content_length))
					self.data = {
						p.get_param("name", header="content-disposition"): p.get_payload(decode=True) for p in m.get_payload()
					}
				else:
					self.data = req["wsgi.input"].read(self.content_length)
			except Exception:
				self._bad_request = True

	@staticmethod
	def _parse_qsa(qs):
		parsed = {}
		for item in qs:
			pair = item.split("=") + [""]
			if pair[0] in parsed:
				if type(parsed[pair[0]]) == list:
					parsed[pair[0]].append(pair[1])
				else:
					parsed[pair[0]] = [parsed[pair[0]], pair[1]]
			else:
				parsed[pair[0]] = pair[1]
		return parsed

	def get_header(self, key):
		return self._headers.get(key)

	def get_bad_request(self):
		return self._bad_request
