from sys import prefix
from orjson import loads
from .route import Route
from .request import Request
from .response import Response


class WSGI:
	def __init__(
		self, auth_fn=None, preparatory_fn=None, cleaning_fn=None,
		content_type="application/json", status_codes={}
	):
		self._auth_fn = auth_fn if callable(auth_fn) else None
		self._content_type = content_type
		self._preparatory_fn = preparatory_fn if callable(preparatory_fn) else None
		self._cleaning_fn = cleaning_fn if callable(cleaning_fn) else None
		if type(status_codes) == dict and type(status_codes.get("text/html")) == dict:
			self.status_codes = status_codes
		else:
			file = open(prefix + "/WeefPy/codes.json", "rb")
			self.status_codes = loads(file.read())
			file.close()

	def __call__(self, env, start_response):
		path = env["PATH_INFO"] if env["PATH_INFO"][-1] == "/" else env["PATH_INFO"] + "/"
		request = Request(env)
		response = Response(start_response, content_type=self._content_type)

		if request.get_bad_request():
			response.set(self.get_code(400), status="400 BAD REQUEST")
		else:
			method, args, method_not_allowed, auth = Route.get(path, env["REQUEST_METHOD"])
			data = None
			if not method:
				if method_not_allowed:
					response.set(self.get_code(405), status="405 METHOD NOT ALLOWED")
				else:
					response.set(self.get_code(404), status="404 NOT FOUND")
			else:
				if self._preparatory_fn:
					self._preparatory_fn(request, response)
				if not auth or self._auth_fn and self._auth_fn(request, response):
					data = method(request, response, *args)
				elif auth and not response.get_status():
					response.set(self.get_code(401), status="401 UNAUTHORIZED")
				if self._cleaning_fn:
					self._cleaning_fn(request, response)
			if data is not None:
				response.set(*data) if type(data) == tuple else response.set(data)
			elif not response.get_status():
				response.set(self.get_code(501), status="501 NOT IMPLEMENTED")
		yield response()

	def get_code(self, code):
		codes = self.status_codes.get(self._content_type, self.status_codes["text/html"])
		if response := codes.get(str(code)):
			return response
		json = "{\"status\": \"\", \"code\": %d, \"message\": \"\"}" % code
		return json if self._content_type == "application/json" else str(code)
